#include <qvaluevector.h>
#include <qobject.h>

#include "cuecontroller.h"
CueController::CueController( DMXEngine* engine, QListView* cueList, 
			      ChannelPanel* panel, QObject* parent = 0, 
			      const char* name = 0 )
  : QObject( parent, name), _engine( engine ), 
    _cueList( cueList ), _panel( panel )
{
  _live = false; 
}


CueController::~CueController()
{
}

void CueController::setLive( bool live )
{
  _live = live;
}


Cue* CueController::getSelectedCue()
{
  if( _cueList->selectedItem() )
    return getCue( _cueList->selectedItem() );
  else
    return NULL;
}

      
Cue* CueController::getCue( QListViewItem* i )
{
  return _cues.find((void*)i );
}

CueListItem* CueController::getCueListItem( Cue* c )
{
  return _cueListItems.find( (void*)c );
}

bool CueController::isRunning( Cue* c )
{
  return false;
}

bool CueController::isActive( Cue* c )
{
  return false;
}


void CueController::selectedCueChanged( QListViewItem* item )
{
  Cue* cue = getCue( item );
  
  if( !_live )
    setPanelView( cue );
}

CueListItem* CueController::newCueItem( Cue* c )
{
  CueListItem* newItem;

  if( _cueList->currentItem() )
    newItem = new CueListItem( _cueList, _cueList->currentItem() );
  else
    if( _cueList->lastItem() )
      newItem = new CueListItem( _cueList, _cueList->lastItem() );
    else
      newItem = new CueListItem( _cueList );

  if( c->followIsSet() && newItem->itemAbove() == NULL )
    c->setFollowTime(0);

  _cues.insert( (void*)newItem, c );
  _cueListItems.insert( (void*)c, newItem );
  
  _cueList->triggerUpdate();

  return newItem;
}

void CueController::recordCueLevels( Cue* currCue, bool allLevels = false )
{
  if( allLevels )
    {

      for( int i = 0; i < _panel->numChannels(); i++ )
	{
	  currCue->clearChannel(i);
	  currCue->setChannel( i, _panel->getChannelLevel( i ) );
	}

      setPanelView( currCue );

    }
  else
    {
      for( int i = 0; i < _engine->getNumFaders(); i++ )
	currCue->clearChannel(i);
      
      Cue* prevCue = getCue( getCueListItem( currCue )->itemAbove() );
  
      if( !prevCue )
	return recordCueLevels( currCue, true );
      
      // set levels for all channels that changed since the last cue.
      
      for( int i = 0; i < _engine->getNumFaders(); i++ )
	if( _panel->getChannelState(i) == CS_CHANGED )
	  {
	    currCue->setChannel( i, _panel->getChannelLevel(i));
	  }

      setPanelView( currCue );

    }
//  printf("CueController::recordCueLevels\n");
}

void CueController::setPanelView( Cue* c )
{
  QValueVector<int> levels( _panel->numChannels() );
  QValueVector<enum ChannelState> states( _panel->numChannels() );

  for(int i = 0; i < _panel->numChannels(); i++ )
    {
      _panel->setRecorded(i, false); 
      levels[i] = 0;
      states[i] = CS_NONE;
    }

  QListViewItem* lastFullItem = (QListViewItem*)getCueListItem(c);
  
  while( getCue(lastFullItem)->numChannels() != _engine->getNumFaders() )
    lastFullItem = lastFullItem->itemAbove();
      
  Cue* currCue = getCue( lastFullItem );
  
  while( currCue != c )
    {

      for( int i = 0; i < _panel->numChannels(); i++ )
	{
	  if( currCue->channelIsSet( i ) )
	    {
	      if( currCue->getChannel(i) > levels[i] )
		states[i] = CS_UP;
	      else if( currCue->getChannel(i) < levels[i] )
		states[i] = CS_DOWN;
	      else if( currCue->getChannel(i) > 0)
		states[i] = CS_SAME;
	      else
		states[i] = CS_NONE;
	    }
	  else if( levels[i] > 0 )
	    states[i] = CS_SAME;
	  else
	    states[i] = CS_NONE;
	}
      
      
      for( currCue->begin(); !currCue->isEnd(); currCue->next() )
	levels[ currCue->currChannel() ] = currCue->currLevel();
      
      currCue = getCue( getCueListItem( currCue )->itemBelow() );
    }

  for( int i = 0; i < _panel->numChannels(); i++ )
    {
      if( currCue->channelIsSet( i ) )
	{
	  if( currCue->getChannel(i) > levels[i] )
	    states[i] = CS_UP;
	  else if( currCue->getChannel(i) < levels[i] )
	    states[i] = CS_DOWN;
	  else if( currCue->getChannel(i) > 0 )
	    states[i] = CS_SAME;
	  else
	    states[i] = CS_NONE;
	}
      else if( levels[i] > 0 )
	states[i] = CS_SAME;
      else
	states[i] = CS_NONE;
    }

  for( currCue->begin(); !currCue->isEnd(); currCue->next() )
    {
      levels[ currCue->currChannel() ] = currCue->currLevel();
      _panel->setRecorded( currCue->currChannel(), true );
    }

  for( int i = 0; i < _panel->numChannels(); i++ )
    {
      _panel->setChannelLevel( i, levels[i] );
      _panel->setChannelState( i, states[i] );
    }

}
