/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/
void editCueForm::accept()
{   

    QString tmpString = numEdit->text();
    int tmpInt = numEdit->cursorPosition();
    if( _doubleValidator->validate( tmpString, tmpInt )
	!= QValidator::Acceptable )
	{
	QMessageBox::warning ( this, "SLight", 
	      "The cue number must be a valid floating point number.", 
	      "OK" );
	
	return;
             }
    
    
    tmpString = fadeEdit->text();
    tmpInt = fadeEdit->cursorPosition();
    if( _doubleValidator->validate( tmpString, tmpInt )
	!= QValidator::Acceptable )
	{
	QMessageBox::warning ( this, "SLight", 
	    "The fade time must be a valid floating point number.", 
	    "OK" );
	
	return;
             }
    
    if( followCheck->isChecked() )
    {
	// there's no way to check if this is the first cue
	// at this point (in which case, a follow time makes
	// no sense).  deal with it later.
	
	tmpString = followEdit->text();
	tmpInt = followEdit->cursorPosition();
	if( _doubleValidator->validate( tmpString, tmpInt )
	    != QValidator::Acceptable && tmpString.toFloat() <= 0 )
	    {
	    QMessageBox::warning ( this, "SLight", 
	       "The follow time must be a valid floating point number.",
	       "OK" );
	    
	    return;
	}
    }
      
    //all inputs are legitimate.  update the cue.
    
    tmpString = nameEdit->text();
    _currCue->setName( tmpString );
    _currCue->setNumber( numEdit->text().toFloat() );
    _currCue->setFadeTime( fadeEdit->text().toFloat() );
    
    if( followCheck->isChecked() )
    {
	_currCue->setFollow(true);
	_currCue->setFollowTime( followEdit->text().toFloat());
    }
    else
	_currCue->setFollow(false);
    
    if( jumpCheck->isChecked() )
    {
	    _currCue->setJumpTimes( jumpSpin->value() );  
	    _currCue->setJumpToCue( model->getCue(
		    jumpCombo->currentItem() ) );
	}
    else
	_currCue->setJumpToCue(NULL);

    QDialog::accept();
//    printf("editform::accept()\n");
}

void editCueForm::init()
{
    
    recordChangedRadio->setChecked(true);
    recordAllRadio->setChecked(false);
    
    _allLevels = false;

    _doubleValidator = new QDoubleValidator( this, "DoubleValidator" );
    
    numEdit->setValidator( _doubleValidator );    
    fadeEdit->setValidator( _doubleValidator );
    followEdit->setValidator( _doubleValidator );
        
    for( unsigned int i = 0; i < model->getNumCues(); i++ )
	jumpCombo->insertItem( model->getCue(i)->name() );   
}	

void editCueForm::destroy()
{
    delete _doubleValidator;
}

bool editCueForm::allLevels()
{
    return _allLevels;
}

void editCueForm::setCue( Cue * c )
{
    _currCue = c;
    
    nameEdit->setText( _currCue->name() );
    numEdit->setText( QString::number(_currCue->number()) );
    fadeEdit->setText( QString::number(_currCue->fadeTime()) );
       
    if( _currCue->followIsSet() )
    {
	followCheck->setChecked(true);
	followEdit->setEnabled(true);
	followEdit->setText( 
		QString::number( _currCue->followTime() ) );
    }
   
    if( _currCue->jumpToCueIsSet() )
    {
	jumpCheck->setChecked(true);
	
	jumpCombo->setEnabled(true);
	jumpCombo->setCurrentItem( 
		model->findCue( _currCue->jumpToCue() ) );
	
	jumpSpin->setEnabled(true);
	jumpSpin->setValue( _currCue->jumpTimes() );
    }
    
    if( _currCue->numChannels() == model->getNumFaders()  )
	recordAllRadio->setChecked(true);
    else
	recordChangedRadio->setChecked(true);
	
}

void editCueForm::toggleFollow( bool f )
{
    if( f )
	followEdit->setEnabled(true);
    else
	followEdit->setEnabled(false);	
}

void editCueForm::toggleJump( bool b )
{
    if( b )
    {
	jumpSpin->setEnabled(true);
	jumpCombo->setEnabled(true);
    }
    else
    {
	jumpSpin->setEnabled( false );
	jumpCombo->setEnabled( false );
    }
}

void editCueForm::toggleAllLevels( bool b )
{
    _allLevels = b;
}
