#include <math.h>

#include "fader.h"
#include "dmxengine.h"

Fader::Fader( unsigned int faderNum, DMXEngine *engine, QObject *parent = 0, 
        const char *name = 0 )
  : QObject( parent, name ), m_faderNum( faderNum ), m_engine( engine )
{
  connect( &m_timer, SIGNAL(timeout()), this, SLOT(timer_tick()) );
  m_level = 0;
  m_targetLevel = 0;
}

Fader::~Fader()
{
}

void Fader::setLevel( int level )
{
  m_timer.stop();
  m_level = (float)level;

  for( QValueList<unsigned int>::iterator i = m_channels.begin();
       i != m_channels.end(); i++ )
      m_engine->effectLevelChange( *i, (int)m_level );
  
  emit levelChanged( m_faderNum, (int)m_level );

}


void Fader::fadeTo( int p_level, float p_time )  //in sec
{
  m_timer.stop();

  if( p_level == m_level )
    return;

  m_targetLevel = p_level;

  int fade_time = (int)floor(p_time * 1000.0);  //in msec

  float wanted_tick_time = fade_time / 
    (float)abs( m_targetLevel - (int)m_level );
  
  int tick_time;
  
  if( wanted_tick_time > m_faderAccuracy )
    {
      tick_time = (int)wanted_tick_time;
      m_stepSize = 1.0;
    }
  else
    {
      tick_time = m_faderAccuracy;
      m_stepSize = (float)m_faderAccuracy / wanted_tick_time;
    }

  m_timer.start( tick_time );
}

void Fader::timer_tick()
{
  if( m_level == m_targetLevel )
    {
      m_timer.stop();
      return;
    }
  
  if( m_level < m_targetLevel )
    {
      if( (m_targetLevel - m_level) < m_stepSize )
	{
	  m_level += m_targetLevel - m_level;
	  m_timer.stop();

          for( QValueList<unsigned int>::iterator i = m_channels.begin();
               i != m_channels.end(); i++ )
            m_engine->effectLevelChange( *i, (int)m_level );

	  emit levelChanged( m_faderNum, (int)m_level );
	  return;
	}
      
      m_level += m_stepSize;
    }
  else
    {
      if( m_level - m_targetLevel < m_stepSize )
	{
	  m_level -= m_level - m_targetLevel;
	  m_timer.stop();

          for( QValueList<unsigned int>::iterator i = m_channels.begin();
               i != m_channels.end(); i++ )
            m_engine->effectLevelChange( *i, (int)m_level );

	  emit levelChanged( m_faderNum, (int)m_level );
	  return;
	}

      m_level -= m_stepSize;
    }
  
  for( QValueList<unsigned int>::iterator i = m_channels.begin();
       i != m_channels.end(); i++ )
      m_engine->effectLevelChange( *i, (int)m_level );

  emit levelChanged( m_faderNum, (int)m_level );

  return;
}



