#include <qmessagebox.h>

#include "maincontroller.h"
#include "mainform.h"
#include "cue.h"
#include "slmodel.h"
#include "slight.h"
#include "zerocue.h"
#include "editform.h"

mainForm* MainController::m_form;
QListView* MainController::m_cueList;
ChannelPanel* MainController::m_faderView;
QPtrDict< CueListItem > MainController::m_cueListItems;
bool MainController::m_live;
Cue* MainController::m_copyCue = NULL;

MainController::MainController( mainForm* form )
{
    m_form = form;
    m_cueList = m_form->getCueList();
    m_faderView = m_form->getFaderView();
}

MainController::~MainController()
{
}


void MainController::clear()
{
    model->clear();
    updateCueList();
    model->goTo( ZeroCue::singleton() );
}

void MainController::restore( QDataStream& s )
{
    m_form->blind(true);
    if( !model->checkFile(s) )
        return;
    model->clear();
    model->restore(s);
    updateCueList();
    model->goTo( ZeroCue::singleton() );
}

void MainController::cut()
{
    // only ever gets called in BLIND
    CueListItem* toCut = (CueListItem*)(m_cueList->selectedItem());
    if( toCut == NULL )
        return;

    if( toCut->getCue() == model->getCurrentCue() )
    {
        QMessageBox::warning( 0, "SLight", 
                QString( "You can't cut a cue currently running in LIVE." ));
        return;
    }

    m_cueList->takeItem( toCut );
    model->removeCue( toCut->getCue() );
    m_cueListItems.remove( toCut->getCue() );

    delete toCut;

    if( m_copyCue != NULL &&
        m_cueListItems.find( m_copyCue ) == NULL )
    {
        delete m_copyCue;
        m_copyCue = NULL;
    }

    m_copyCue = toCut->getCue();
    m_cueList->triggerUpdate();
}

void MainController::copy()
{
    // only ever gets called in BLIND

    CueListItem* toCopy = (CueListItem*)(m_cueList->selectedItem());
    if( toCopy == NULL )
        return;

    if( m_copyCue != NULL && 
        m_cueListItems.find( m_copyCue ) == NULL )
    {
        delete m_copyCue;
        m_copyCue = NULL;
    }

    m_copyCue = toCopy->getCue();
}

void MainController::paste()
{
    // only ever gets called in BLIND

    if( m_copyCue == NULL )
        return;

    CueListItem* insertAfter = (CueListItem*)(m_cueList->selectedItem());
    if( insertAfter == NULL )
        return;

    Cue* newCue = new Cue( *m_copyCue );
    model->insertCue( newCue, insertAfter->getCue() );
    newCueListItem( newCue );
    model->checkpoint();
}

void MainController::deleteCue()
{
    CueListItem* cueItem = (CueListItem*)(m_cueList->selectedItem());
    if( cueItem == NULL )
    {
        return;
    }

    if( cueItem->getCue() == ZeroCue::singleton() )
    {
        QMessageBox::warning( 0, "SLight",
                "You can't delete the blackout cue!" );
        return;
    }

    if( cueItem->getCue() == model->getCurrentCue() )
    {
        QMessageBox::warning( 0, "SLight",
                "You can't delete a cue currently running in LIVE." );
        return;
    }

    if( QMessageBox::warning( 0, "SLight", 
                "Do you want to delete this cue?", 
                QMessageBox::Yes, QMessageBox::No )
            == QMessageBox::No )
        return;

    Cue* cue = cueItem->getCue();
    Cue* prevCue = model->getCue( model->findCue(cue) - 1 );
    CueListItem* prevCueListItem = getCueListItem( prevCue );

    m_cueList->takeItem(cueItem);
    delete cueItem;

    model->removeCue( cue );
    delete cue;

    m_cueList->setSelected( prevCueListItem, true );

    return;
}

void MainController::updateCueList()
{
    
    m_cueList->clear();
    m_cueListItems.clear();

    for(unsigned int i = 0; i < model->getNumCues(); i++ )
    {
        m_cueList->setCurrentItem( m_cueList->lastItem() );
        newCueListItem( model->getCue(i) );
    }

    m_cueList->setSelected( m_cueList->firstChild(), true );

    m_cueList->triggerUpdate();

}

CueListItem* MainController::newCueListItem( Cue* p_cue )
{
    CueListItem* newItem;

    if( m_live )
    {
        newItem = new CueListItem( p_cue, m_cueList,
               getCueListItem( model->getCurrentCue() ) );
    }
    else
    {
        if( m_cueList->selectedItem() )
            newItem = new CueListItem( p_cue, m_cueList, 
                    m_cueList->selectedItem() );
        else
            newItem = new CueListItem( p_cue, m_cueList,
                    m_cueList->lastItem());
    }

    /*
    else
        if( m_cueList->lastItem() )
            newItem = new CueListItem( p_cue, m_cueList, 
                    m_cueList->lastItem() );
        else
            newItem = new CueListItem( p_cue, m_cueList );
            */

    if( p_cue->followIsSet() && newItem->itemAbove() == NULL )
        p_cue->setFollowTime(0);

    /*
    newItem->setText( 0, QString( "Cue %1: %2")
                          .arg(p_cue->number())
                          .arg(p_cue->name()) );

                          */

    m_cueListItems.insert( (void*) p_cue, newItem );

    return newItem;
}

CueListItem* MainController::getCueListItem( Cue* p_cue )
{
    return m_cueListItems.find( (void*)p_cue );
}


void MainController::editCue(Cue* cue)
{
    editCueForm editForm;
    editForm.setCue( cue );
    editForm.exec();

    m_cueList->triggerUpdate();
}
