#ifndef _maincontroller_h
#define _maincontroller_h

#include <qlistview.h>
#include <qptrdict.h>

#include "cue.h"
#include "slmodel.h"
#include "slight.h"
#include "widgets/cuelistitem.h"
#include "widgets/channelpanel.h"

class mainForm;
class QKeyEvent;

class MainController
{

    public:

    MainController( mainForm* form );
    virtual ~MainController();

    static void clear();
    static void restore( QDataStream& s );

    static void cut();
    static void copy();
    static void paste();
    static void deleteCue();
    static void editCue( Cue *cue );

    virtual void fadeToCue( Cue *cue ) = 0;

    virtual void setup() = 0;
    virtual void teardown() = 0;

    virtual void cueDoubleClicked( QListViewItem* p_item ) = 0;
    virtual void cueSelected( QListViewItem* p_item ) = 0;

    virtual void recordClicked() = 0;
    virtual void recordNewClicked() = 0;

    virtual void numFadersChanged( uint p_numFaders ) = 0;
    virtual void modelLevelChanged( uint p_fader, int p_level ) = 0;
    virtual void panelLevelChanged( uint p_fader, int p_level ) = 0;

    virtual void keyPressEvent( QKeyEvent* p_event ) = 0;
    virtual void keyReleaseEvent( QKeyEvent* p_event ) = 0;
    virtual void cueContextMenu( QListViewItem* item, const QPoint& pos ) = 0;

    protected:

    static CueListItem* newCueListItem( Cue* p_cue );
    static CueListItem* getCueListItem( Cue* p_cue );

    static void updateCueList();

    static mainForm* m_form;
    static QListView* m_cueList;
    static ChannelPanel* m_faderView;

    static QPtrDict<CueListItem> m_cueListItems;
    static bool m_live;

    private:
    static Cue* m_copyCue;
};

class MainControllerBlind;
class MainControllerLive;
#endif

