#include <qmessagebox.h>
#include <qvaluevector.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qnamespace.h>
#include <qaction.h>

#include "zerocue.h"
#include "maincontrollerblind.h"
#include "editform.h"

MainControllerBlind::MainControllerBlind( mainForm* form ) 
    : MainController( form ), m_selectedCue( ZeroCue::singleton() ),
    m_cueContextMenu( form, "Cue context menu" )
{
    /* only ever gets called once. */

    model->appendCue( ZeroCue::singleton() );
    m_selectedCue = ZeroCue::singleton();
    CueListItem* newItem = new CueListItem( ZeroCue::singleton(),
            m_cueList );
    m_cueListItems.insert( (void*) ZeroCue::singleton(), newItem );

    // setup the context menu

    m_cueContextMenu.insertItem( QString("Edit cue..."),  m_form,
            SLOT(editCue()) );
    m_cueContextMenu.insertItem( QString("Cut"), m_form, 
            SLOT(editCut()) );
    m_cueContextMenu.insertItem( QString("Copy"), m_form, 
            SLOT(editCopy()) );
    m_cueContextMenu.insertItem( QString("Paste"),  m_form, 
            SLOT(editPaste()) );
    m_cueContextMenu.insertItem( QString("Delete"), m_form,
            SLOT(deleteCue()) );
}

MainControllerBlind::~MainControllerBlind()
{
}

void MainControllerBlind::setup()
{
    m_cueList->setSelectionMode( QListView::Single );

    /*
    if( m_cueList->selectedItem() )
        m_cueList->setSelected( m_cueList->selectedItem(), false );
        */

    if( m_selectedCue )
    {
        m_cueList->setSelected( getCueListItem( m_selectedCue ), true );
        m_cueList->ensureItemVisible( getCueListItem( m_selectedCue ) );
    }

    m_live = false;
}

void MainControllerBlind::teardown()
{
    m_selectedCue = ((CueListItem*)(m_cueList->currentItem()))->getCue();
}

void MainControllerBlind::cueDoubleClicked( QListViewItem* p_item )
{
    Cue* cue = ((CueListItem*)p_item)->getCue();

    editCue(cue);
/*
    editCueForm editForm;
    editForm.setCue( cue );
    editForm.exec();

    m_cueList->triggerUpdate();
*/
    /*
    if( editForm.result() == QDialog::Accepted )
    {
       p_item->setText(0, QString( "Cue %1: %2")
                          .arg(cue->number())
                          .arg(cue->name()) );
        if( cue->jumpToCueIsSet() )
    }
    */
}

void MainControllerBlind::cueSelected( QListViewItem* p_item )
{

    m_cueList->ensureItemVisible( p_item );

    Cue* cue = ((CueListItem*)(p_item))->getCue();
    if( model->findCue(cue) != -1 )
        setPanelView( cue );
}

void MainControllerBlind::recordClicked()
{
    CueListItem* selectedItem = ((CueListItem*)m_cueList->selectedItem());

    if( selectedItem != NULL ) {
        recordCueLevels( selectedItem->getCue(), false );
	model->checkpoint();
    }
    else
        QMessageBox::information( NULL, "SLight", 
                "You must select a cue to record." );
}

void MainControllerBlind::recordNewClicked()
{
    m_faderView->cancelSelection();
    Cue* cue = new Cue;

    editCueForm editForm;
    editForm.setCue( cue );
    editForm.exec();

    if( editForm.result() == QDialog::Accepted )
    {
        CueListItem* item = newCueListItem( cue );

        CueListItem* aboveItem = (CueListItem*)(item->itemAbove());

        if( aboveItem )
            model->insertCue( cue, aboveItem->getCue() );
        else
            model->insertCue( cue, (unsigned int)0 );

        recordCueLevels( cue, editForm.allLevels() );

        if( m_cueList->selectedItem() )
            m_cueList->setSelected( m_cueList->selectedItem(), false );
        
        m_cueList->setSelected(item, true);

        m_cueList->triggerUpdate();
	model->checkpoint();
    }
    else
        delete cue;
}

void MainControllerBlind::numFadersChanged( uint p_numFaders )
{
    if( p_numFaders != m_faderView->numChannels() )
        m_faderView->setNumChannels( p_numFaders );
}

void MainControllerBlind::modelLevelChanged( uint, int )
{
    // do nothing.
}

void MainControllerBlind::panelLevelChanged( uint, int )
{
    // do nothing.
}

void MainControllerBlind::keyPressEvent( QKeyEvent* p_event )
{
    // remember:  p_event's accept flag is set by default!  if you want
    // the keypress, just return.  if not, call p_event->ignore() and
    // then return

    CueListItem* cueItem = (CueListItem*)(m_cueList->currentItem());

    switch( p_event->key() )
    {
        case Qt::Key_Delete:

            if( m_faderView->isAnySelected() )
            {               
                if( cueItem == NULL )
                {
                    p_event->ignore();
                    return;
                }

                Cue* cue = cueItem->getCue();

                if( QMessageBox::warning( 0, "SLight", 
                            "Do you want to delete these channels from this cue?", 
                            QMessageBox::Yes, QMessageBox::No )
                        == QMessageBox::No )
                    return;

                for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
                    if( m_faderView->isSelected( i ) )
                        cue->clearChannel(i);

                setPanelView(cue);

                return;
            }
            else
            {
                m_form->deleteCue();
                return;
            }
                /*
            {
                if( cueItem == NULL )
                {
                    p_event->ignore();
                    return;
                }

                if( cueItem->getCue() == ZeroCue::singleton() )
                {
                    QMessageBox::warning( 0, "SLight",
                            "You can't delete the blackout cue!" );
                    return;
                }

                if( cueItem->getCue() == model->getCurrentCue() )
                {
                    QMessageBox::warning( 0, "SLight",
                            "You can't delete a cue currently running in LIVE." );
                    return;
                }

                if( QMessageBox::warning( 0, "SLight", 
                            "Do you want to delete this cue?", 
                            QMessageBox::Yes, QMessageBox::No )
                        == QMessageBox::No )
                    return;

                Cue* cue = cueItem->getCue();
                Cue* prevCue = model->getCue( model->findCue(cue) - 1 );
                CueListItem* prevCueListItem = getCueListItem( prevCue );

                m_cueList->takeItem(cueItem);
                delete cueItem;

                model->removeCue( cue );
                delete cue;

                m_cueList->setSelected( prevCueListItem, true );

                return;
            }
            */

        case Qt::Key_Escape:
            if( m_faderView->isAnySelected() )
            {
                // want this to be taken care of by the widget's handler
                p_event->ignore();
                return;
            }
            else
            {
                if( cueItem == NULL )
                {
                    p_event->ignore();
                    return;
                }
                Cue* cue = cueItem->getCue();
                setPanelView( cue );
                return;
            }

        case Qt::Key_Up:
            if( cueItem == NULL )
                {
                    p_event->ignore();
                    return;
                }

            if( cueItem->itemAbove() )
                m_cueList->setSelected( cueItem->itemAbove(), true );
         
            return;

        case Qt::Key_Down:
            if( cueItem == NULL )
                {
                    p_event->ignore();
                    return;
                }

             if( cueItem->itemBelow() )
                m_cueList->setSelected( cueItem->itemBelow(), true );
         
             return;
    case Qt::Key_F1:
    case Qt::Key_F2:
    case Qt::Key_F3:
    case Qt::Key_F4:
    case Qt::Key_F5:
    case Qt::Key_F6:
    case Qt::Key_F7:
    case Qt::Key_F8:
	    printf("blind pressed F%d\n", p_event->key() - Qt::Key_F1 + 1);
	    return;

        default:

            p_event->ignore();
            return;
    }
}

void MainControllerBlind::keyReleaseEvent( QKeyEvent* p_event )
{
    switch( p_event->key() )
    {
    case Qt::Key_F1:
    case Qt::Key_F2:
    case Qt::Key_F3:
    case Qt::Key_F4:
    case Qt::Key_F5:
    case Qt::Key_F6:
    case Qt::Key_F7:
    case Qt::Key_F8:
	    printf("blind released F%d\n", p_event->key() - Qt::Key_F1 + 1);
	    return;

    default:
            p_event->ignore();
            return;
    }
}

void MainControllerBlind::cueContextMenu( QListViewItem*, const QPoint& pos )
{
    m_cueContextMenu.exec(pos);
}

void MainControllerBlind::recordCueLevels( Cue* p_cue, bool p_allLevels )
{
    if( p_allLevels )
    {

        for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
        {
            p_cue->clearChannel(i);
            p_cue->setChannel( i, m_faderView->getChannelLevel( i ) );
        }

        setPanelView( p_cue );

        return;
    }
    else
    {
        CueListItem* prevItem = (CueListItem*)(getCueListItem( p_cue )->itemAbove());
        Cue* prevCue = NULL;

        if( !prevItem )
            return recordCueLevels( p_cue, true );
        else
            prevCue = prevItem->getCue();

        // set levels for all channels that changed since the last cue.

        for( unsigned int i = 0; i < model->getNumFaders(); i++ )
            if( m_faderView->getChannelState(i) == CS_CHANGED )
            {
                p_cue->setChannel( i, m_faderView->getChannelLevel(i));
            }

        setPanelView( p_cue );

        return;
    }
}

void MainControllerBlind::setPanelView( Cue* p_cue )
{
    QValueVector<int> levels( m_faderView->numChannels() );
    QValueVector<enum ChannelState> states( m_faderView->numChannels() );

    m_faderView->cancelSelection();

    for(unsigned int i = 0; i < m_faderView->numChannels(); i++ )
    {
        m_faderView->setRecorded(i, false); 
        levels[i] = 0;
        states[i] = CS_NONE;
    }

    int lastFullCueIndex = model->findCue(p_cue);

    while( model->getCue(lastFullCueIndex)->numChannels() 
            != model->getNumFaders() )
        lastFullCueIndex--;

    Cue* currCue = model->getCue( lastFullCueIndex );

    while( currCue != p_cue )
    {
        for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
        {
            if( currCue->channelIsSet( i ) )
            {
                if( currCue->getChannel(i) > levels[i] )
                    states[i] = CS_UP;
                else if( currCue->getChannel(i) < levels[i] )
                    states[i] = CS_DOWN;
                else if( currCue->getChannel(i) > 0)
                    states[i] = CS_SAME;
                else
                    states[i] = CS_NONE;
            }
            else if( levels[i] > 0 )
                states[i] = CS_SAME;
            else
                states[i] = CS_NONE;
        }


        for( currCue->begin(); currCue->hasNext(); currCue->next() )
            levels[ currCue->currChannel() ] = currCue->currLevel();

        currCue = model->getCue( model->findCue(currCue) + 1 );
    }

    for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
    {
        if( currCue->channelIsSet( i ) )
        {
            if( currCue->getChannel(i) > levels[i] )
                states[i] = CS_UP;
            else if( currCue->getChannel(i) < levels[i] )
                states[i] = CS_DOWN;
            else if( currCue->getChannel(i) > 0 )
                states[i] = CS_SAME;
            else
                states[i] = CS_NONE;
        }
        else if( levels[i] > 0 )
            states[i] = CS_SAME;
        else
            states[i] = CS_NONE;
    }

    for( currCue->begin(); currCue->hasNext(); currCue->next() )
    {
        levels[ currCue->currChannel() ] = currCue->currLevel();
        m_faderView->setRecorded( currCue->currChannel(), true );
    }

    for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
    {
        m_faderView->setChannelLevel( i, levels[i] );
        m_faderView->setChannelState( i, states[i] );
    }

}

void MainControllerBlind::fadeToCue(Cue* cue)
{
	printf("MainControllerBlind::fadeToCue: can't do that in blind");
}
