#include <qmessagebox.h>
#include <qpopupmenu.h>

#include "mainform.h"
#include "editform.h"
#include "maincontrollerlive.h"
#include "zerocue.h"

MainControllerLive::MainControllerLive( mainForm* form ) 
    : MainController( form ) 
{
    model->goTo( ZeroCue::singleton() );

    // setup the context menu

    m_cueContextMenu.insertItem( QString("Edit cue..."),  m_form,
            SLOT(editCue()) );
    m_cueContextMenu.insertSeparator( );
    m_cueContextMenu.insertSeparator( );
    m_cueContextMenu.insertSeparator( );
    m_cueContextMenu.insertSeparator( );

    m_cueContextMenu.insertItem( QString("Fade to cue"),  m_form,
            SLOT(fadeToCue()) );
}

MainControllerLive::~MainControllerLive()
{
}

void MainControllerLive::setup()
{
    if( m_cueList->selectedItem() )
        m_cueList->setSelected( m_cueList->selectedItem(), false );

    m_cueList->setSelectionMode( QListView::NoSelection );
    m_cueList->ensureItemVisible( getCueListItem( model->getCurrentCue() ) );

    for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
    {
        m_faderView->setRecorded( i, false );
        m_faderView->setChannelLevel( i, model->getFaderLevel(i) );

        if( i < m_savedLevels.size() )
        {
            //m_faderView->setChannelLevel( i, m_savedLevels[i] );
            //m_faderView->setChannelLevel( i, model->getFaderLevel(i) );
            m_faderView->setChannelState( i, m_savedStates[i] );
        }
        else
            m_faderView->setChannelState( i, CS_NONE );

    }

    m_live = true;
}

void MainControllerLive::teardown()
{
    m_savedLevels.resize( model->getNumFaders() );
    m_savedStates.resize( model->getNumFaders() );

    for( unsigned int i = 0; i < model->getNumFaders(); i++ )
    {
        m_savedLevels[i] = m_faderView->getChannelLevel(i);
        m_savedStates[i] = m_faderView->getChannelState(i);
    }
}

void MainControllerLive::cueDoubleClicked( QListViewItem* p_item )
{
    Cue* cue = ((CueListItem*)p_item)->getCue();
    model->goTo( cue );
}

void MainControllerLive::cueSelected( QListViewItem* p_item)
{
/*	Cue* cue = ((CueListItem*)(p_item))->getCue();
	printf("MainControllerLive::cueSelected %.1f (%s)\n", 
	       cue->number(), 
	       (char *)cue->name().ascii());
*/	
    //nothing happens.    
}

void MainControllerLive::recordClicked()
{
    if( model->getCurrentCue() == ZeroCue::singleton() )
    {
        QMessageBox::information( NULL, "SLight", 
                "You can't change the levels of the first blackout cue!" );
        return;
    }

    recordCueLevels( model->getCurrentCue(), false );
    model->checkpoint();
}

void MainControllerLive::recordNewClicked()
{
    m_faderView->cancelSelection();
    Cue* cue = new Cue;

    editCueForm editForm;
    editForm.setCue( cue );
    editForm.exec();

    if( editForm.result() == QDialog::Accepted )
    {
        /*
        CueListItem* item = newCueListItem( cue );

        CueListItem* aboveItem = (CueListItem*)(item->itemAbove());

        if( aboveItem )
            model->insertCue( cue, aboveItem->getCue() );
        else
            model->insertCue( cue, (unsigned int)0 );
            */

        newCueListItem( cue );
        model->insertCue( cue, model->getCurrentCue() );
        recordCueLevels( cue, editForm.allLevels() );

        model->goTo( cue );
                
        //m_cueList->setSelected(item, true);
	model->checkpoint();
    }
    else
        delete cue;

}

void MainControllerLive::numFadersChanged( uint )
{
    //never happens.
}

void MainControllerLive::modelLevelChanged( uint p_fader, int p_level )
{
    if( p_level > m_faderView->getChannelLevel( p_fader ) )
        m_faderView->setChannelState( p_fader, CS_UP );
    else if( p_level < m_faderView->getChannelLevel( p_fader ) )
        m_faderView->setChannelState( p_fader, CS_DOWN );
   /*
    else if( p_level != 0 )
        m_faderView->setChannelState( p_fader, CS_SAME );
    else
        m_faderView->setChannelState( p_fader, CS_NONE );
        */

    m_faderView->setChannelLevel( p_fader, p_level );
}

void MainControllerLive::panelLevelChanged( uint p_fader, int p_level )
{
    model->blockSignals(true);

    
    if( p_level > model->getFaderLevel( p_fader ) )
        m_faderView->setChannelState( p_fader, CS_UP );
    else if( p_level < model->getFaderLevel( p_fader ) )
        m_faderView->setChannelState( p_fader, CS_DOWN );
    /*
    else if( p_level != 0 )
        m_faderView->setChannelState( p_fader, CS_SAME );
    else
        m_faderView->setChannelState( p_fader, CS_NONE );
        */

    model->setFaderLevel( p_fader, p_level );

    model->blockSignals(false);
}

void MainControllerLive::keyPressEvent( QKeyEvent* p_event )
{
    CueListItem* cueItem = (CueListItem*)(m_cueList->currentItem());
    switch( p_event->key() )
    {
        case Qt::Key_Space:
            for( unsigned int i = 0; i < model->getNumFaders(); i++ )
            {
                if( m_faderView->getChannelLevel( i ) == 0 )
                    m_faderView->setChannelState( i, CS_NONE );
                else
                    m_faderView->setChannelState( i, CS_SAME );
            }

            model->fadeToNextCue(); 
            m_cueList->ensureItemVisible( 
                    getCueListItem( model->getCurrentCue() ) );
            return;

        case Qt::Key_Up:
            m_cueList->scrollBy( 0, -1 * m_cueList->firstChild()->height() );
            return;

        case Qt::Key_Down:
            m_cueList->scrollBy( 0, m_cueList->firstChild()->height() );
            return;

        case Qt::Key_Delete:

            if( m_faderView->isAnySelected() )
            {               
                if( cueItem == NULL )
                {
                    p_event->ignore();
                    return;
                }

                Cue* cue = cueItem->getCue();
/*		printf("cue %.1f (%s)\n", cue->number(), 
  (char *)cue->name().ascii()); */

                if( QMessageBox::warning( 0, "SLight", 
                            "Do you want to delete these channels from this cue?", 
                            QMessageBox::Yes, QMessageBox::No )
                        == QMessageBox::No )
                    return;

                for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
			if( m_faderView->isSelected( i ) ) {
//				printf("channel %d\n", i);
				cue->clearChannel(i);
			}
//		cue->toascii(stdout,true);
                setPanelView(cue);
		model->checkpoint();

                return;
            }
    case Qt::Key_F1:
    case Qt::Key_F2:
    case Qt::Key_F3:
    case Qt::Key_F4:
    case Qt::Key_F5:
    case Qt::Key_F6:
    case Qt::Key_F7:
    case Qt::Key_F8:
    {
	    int fader = p_event->key() - Qt::Key_F1;
	    //printf("pressed F%d\n", fader+1);
	    model->setFaderLevel(fader , 217 );
	    return;
    }
    default:
            p_event->ignore();
            return;
    }
}

void MainControllerLive::keyReleaseEvent( QKeyEvent* p_event )
{
    switch( p_event->key() )
    {
    case Qt::Key_F1:
    case Qt::Key_F2:
    case Qt::Key_F3:
    case Qt::Key_F4:
    case Qt::Key_F5:
    case Qt::Key_F6:
    case Qt::Key_F7:
    case Qt::Key_F8:
    {
	    int fader = p_event->key() - Qt::Key_F1;
	    Cue *cue = model->getCurrentCue();
	    int level = model->getFaderForCue(cue, fader);
	    //printf("released F%d\n", fader+1);
	    model->setFaderLevel(fader , level );
	    return;
    }

    default:
            p_event->ignore();
            return;
    }
}

Cue* g_context_cue;

void MainControllerLive::cueContextMenu( QListViewItem* p_item, 
					 const QPoint& pos)
{
//	printf("MainControllerLive::cueContextMenu item=%p\n", p_item);
	Cue* cue;
	if(p_item) {
		cue = ((CueListItem*)(p_item))->getCue();
		/*printf("MainControllerLive::cueContext %.1f (%s)\n", 
		       cue->number(), 
		       (char *)cue->name().ascii());*/
		g_context_cue = cue;
	} else {
		g_context_cue = NULL;
	}

	m_cueContextMenu.exec(pos);
}

void MainControllerLive::recordCueLevels( Cue* p_cue, bool p_allLevels )
{

    if( p_allLevels )
    {
        for( unsigned int i = 0; i < model->getNumFaders(); i++ )
        {
            p_cue->clearChannel(i);
            p_cue->setChannel( i, model->getFaderLevel(i));
        }
    }
    else
    {

        QValueVector<int> levels( m_faderView->numChannels() );

        int lastFullCueIndex = model->findCue(p_cue);

        while( model->getCue(lastFullCueIndex)->numChannels() 
                != model->getNumFaders() )
            lastFullCueIndex--;

        Cue* currCue = model->getCue( lastFullCueIndex );
        while( currCue != p_cue )
        {
            for( currCue->begin(); currCue->hasNext(); currCue->next() )
                levels[ currCue->currChannel() ] = currCue->currLevel();

            currCue = model->getCue( model->findCue(currCue) + 1 );
        }

        for( currCue->begin(); currCue->hasNext(); currCue->next() )
            levels[ currCue->currChannel() ] = currCue->currLevel();

        //levels are now set to what they were *before* we began goofing with them.

        for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
            if( m_faderView->getChannelLevel(i) != levels[i] )
                p_cue->setChannel( i, m_faderView->getChannelLevel(i) );
    }
}


/* This is the same as MainControllerBlind::setPanelView  - can't we share?
 */
void MainControllerLive::setPanelView( Cue* p_cue )
{
    QValueVector<int> levels( m_faderView->numChannels() );
    QValueVector<enum ChannelState> states( m_faderView->numChannels() );

    m_faderView->cancelSelection();

    for(unsigned int i = 0; i < m_faderView->numChannels(); i++ )
    {
        m_faderView->setRecorded(i, false); 
        levels[i] = 0;
        states[i] = CS_NONE;
    }

    int lastFullCueIndex = model->findCue(p_cue);

    while( model->getCue(lastFullCueIndex)->numChannels() 
            != model->getNumFaders() )
        lastFullCueIndex--;

    Cue* currCue = model->getCue( lastFullCueIndex );

    while( currCue != p_cue )
    {
        for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
        {
            if( currCue->channelIsSet( i ) )
            {
                if( currCue->getChannel(i) > levels[i] )
                    states[i] = CS_UP;
                else if( currCue->getChannel(i) < levels[i] )
                    states[i] = CS_DOWN;
                else if( currCue->getChannel(i) > 0)
                    states[i] = CS_SAME;
                else
                    states[i] = CS_NONE;
            }
            else if( levels[i] > 0 )
                states[i] = CS_SAME;
            else
                states[i] = CS_NONE;
        }


        for( currCue->begin(); currCue->hasNext(); currCue->next() )
            levels[ currCue->currChannel() ] = currCue->currLevel();

        currCue = model->getCue( model->findCue(currCue) + 1 );
    }

    for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
    {
        if( currCue->channelIsSet( i ) )
        {
            if( currCue->getChannel(i) > levels[i] )
                states[i] = CS_UP;
            else if( currCue->getChannel(i) < levels[i] )
                states[i] = CS_DOWN;
            else if( currCue->getChannel(i) > 0 )
                states[i] = CS_SAME;
            else
                states[i] = CS_NONE;
        }
        else if( levels[i] > 0 )
            states[i] = CS_SAME;
        else
            states[i] = CS_NONE;
    }

    for( currCue->begin(); currCue->hasNext(); currCue->next() )
    {
        levels[ currCue->currChannel() ] = currCue->currLevel();
        m_faderView->setRecorded( currCue->currChannel(), true );
    }

    for( unsigned int i = 0; i < m_faderView->numChannels(); i++ )
    {
        m_faderView->setChannelLevel( i, levels[i] );
        m_faderView->setChannelState( i, states[i] );
    }

}


void MainControllerLive::fadeToCue(Cue* cue)
{
	for( unsigned int i = 0; i < model->getNumFaders(); i++ )
	{
                if( m_faderView->getChannelLevel( i ) == 0 )
			m_faderView->setChannelState( i, CS_NONE );
                else
			m_faderView->setChannelState( i, CS_SAME );
	}

	model->fadeToCue(cue); 

	m_cueList->ensureItemVisible( 
		getCueListItem( model->getCurrentCue() ) );
}
