/**************************************************************************** ** ui.h extension file, included from the uic-generated form implementation. ** ** If you wish to add, delete or rename slots use Qt Designer which will ** update this file, preserving your code. Create an init() slot in place of ** a constructor, and a destroy() slot in place of a destructor. *****************************************************************************/  
#include <errno.h>


void mainForm::init()
{     
    connect( model, SIGNAL( numFadersChanged(uint) ),
	     this, SLOT( numFadersChanged(uint) ) );
    
    connect( model, SIGNAL( cueStatusChanged() ),
	     cueList, SLOT( triggerUpdate() ) );
    
    connect( faderView, SIGNAL( levelRecorded( uint, int ) ),
	     this, SLOT( panelLevelChanged(uint, int) ) );
    
    connect( resetAction, SIGNAL( activated() ),
	     model, SLOT( reset() ) );
    
    _live = false;
    
    _blindController = new MainControllerBlind( this );
    _liveController = new MainControllerLive( this );
    
    controller = _blindController;    
    controller->setup(); 
    resetAction->setEnabled(false);
    
    setFocusPolicy(QWidget::NoFocus);
    cueList->setFocusPolicy( QWidget::NoFocus );
    faderView->setFocusPolicy( QWidget::NoFocus );
    
    setFocus();
    
    cueList->setSorting(-1);
    
    model->setNumFaders( INITIAL_NUM_CHANNELS );
    model->setNumChannels( INITIAL_NUM_CHANNELS ); 
    for( unsigned int i = 0; i < INITIAL_NUM_CHANNELS; i++ )
	model->mapChannelToFader( i, i );
    
    _filename = QString::null;
}   

void mainForm::fileNew() 
{
    controller->clear();
}  
void mainForm::fileOpen() 
{
    QString filename = QFileDialog::getOpenFileName(
	    QString::null,
	    "SLight Cue Lists (*.sli)",
	    this, 
	    "open file dialog",
	    "Choose a file" );
    
    if(filename != QString::null )
    {
	_filename = filename;
	QFile f( _filename );
	if ( f.open(IO_ReadOnly) ) 
	{   
	    // file opened successfully
	    QDataStream s( &f );        // use a text stream
	    controller->restore(s);
	    f.close();
	}
    }
}  
void mainForm::fileSave() 
{
    if( _filename == QString::null )
    {
	fileSaveAs();
	return;
    }
     
    QFile f( _filename );
    if ( f.open(IO_WriteOnly) ) 
    {   
	// file opened successfully
	QDataStream s( &f );        // use a text stream
	model->serialize( s );
	f.close();
    } else {
	    int e = errno;
	    perror(_filename);
	    QMessageBox::warning( 0, "SLight",
		  QString( "Error: Unable to save \"" ) + _filename + "\":\n"
				  + strerror(e));
    }
    char *afn = new char[strlen(_filename)+6];
    sprintf(afn, "%s.asc", (const char *)_filename);
    FILE *fp;
    if((fp = fopen(afn, "w")) != NULL) {
	    model->toascii(fp);
	    fclose(fp);
    } else {
	    int e = errno;
	    perror(afn);
	    QMessageBox::warning( 0, "SLight",
		  QString( "Error: Unable to save \"" ) + afn + "\":\n"
				  + strerror(e));
    }
 
}  

void mainForm::fileSaveAs()
{
    QString filename = 
	    QFileDialog::getSaveFileName( QString::null,
					  "SLight Cue Lists (*.sli)",
					  this,
					  "save file dialog",
					  "Choose a file" );
    
    // filename == QString::null if "Cancel" was selected
    if( filename != QString::null )
    {
	_filename = filename;
	fileSave();
    }
}  
void mainForm::filePrint() {  }  
void mainForm::fileExit() 
{
    qApp->quit();
}  
void mainForm::editUndo() {  }  
void mainForm::editRedo() {  }  

void mainForm::blind(bool b) 
{  
    if( b && _live )
    {
	_live = false;
	liveButton->setOn(false);
	blindButton->setOn(true);
	
	//recordAction->setEnabled(true);
	//recordNewCueAction->setEnabled(true);
	editCutAction->setEnabled(true);
	editCopyAction->setEnabled(true);
	editPasteAction->setEnabled(true);
	showPatchAction->setEnabled(true);
	resetAction->setEnabled(false);
	editCueAction->setEnabled(true);
	deleteCueAction->setEnabled(true);
	
	controller->teardown();
	controller = _blindController;
	controller->setup();
    }
    else if( !_live )
	blindButton->setOn( true );
}    

void mainForm::editPaste() 
{  
    controller->paste();
}  

void mainForm::editCut() 
{
    controller->cut();
}  

void mainForm::editCopy()
{
    controller->copy();
}

void mainForm::editFind() 
{
}  

void mainForm::helpIndex() 
{
    QMessageBox::information( this, "SLight", QString("SLight has no online help at the moment.\n")
				  + "Please read the accompanying documentation.");
}  

void mainForm::helpContents() 
{  
    QMessageBox::information( this, "SLight", QString("SLight has no online help at the moment.\n")
				  + "Please read the accompanying documentation.");
}
void mainForm::helpAbout() 
{
    QMessageBox::information( this, "SLight", QString("SLight\n") + "Version 0.95\n" 
			      + "Brian Teague\n" + "bteague@rice.edu" );
}  

void mainForm::showPatchPanel()
{     
    patchPanelForm* f = new patchPanelForm;
    f->exec();
}   

void mainForm::live( bool b ) 
{
    if( b && !_live )
    {
	_live = true;
	blindButton->setOn(false);
	liveButton->setOn(true);
	
	//recordAction->setEnabled(false);
	//recordNewCueAction->setEnabled(false);
	editCutAction->setEnabled(false);
	editCopyAction->setEnabled(false);
	editPasteAction->setEnabled(false);
	showPatchAction->setEnabled(false);
	resetAction->setEnabled(true);
//	editCueAction->setEnabled(false);
	editCueAction->setEnabled(true);
	deleteCueAction->setEnabled(false);
	
	controller->teardown();
	controller = _liveController;
	controller->setup();
    }
    else if( _live )
	liveButton->setOn(true);
}  

void mainForm::destroy()
{     
    delete _blindController;
    delete _liveController;
}  

void mainForm::record() 
{     
    controller->recordClicked();
}  

void mainForm::recordNew()
{  
    controller->recordNewClicked();
}

void mainForm::cueDoubleClicked( QListViewItem * i )
{
    controller->cueDoubleClicked( i );
}

void mainForm::cueSelected( QListViewItem * i )
{
    controller->cueSelected(i);
}

void mainForm::modelLevelChanged( uint fader, int level )
{
    controller->modelLevelChanged( fader, level );
}

void mainForm::panelLevelChanged( uint fader, int level )
{
    controller->panelLevelChanged( fader, level );
}

void mainForm::numFadersChanged( uint num )
{
    controller->numFadersChanged( num );
}


QListView* mainForm::getCueList()
{
    return cueList;
}

ChannelPanel* mainForm::getFaderView()
{
    return faderView;
}

void mainForm::keyPressEvent( QKeyEvent* e )
{
    controller->keyPressEvent( e );
    
    if( !e->isAccepted() )
    {
	e->accept();
	faderView->keyPress( e );
    }
}

void mainForm::keyReleaseEvent( QKeyEvent* e )
{
	controller->keyReleaseEvent( e );
	if( !e->isAccepted() )
	{
		e->accept();
	}
}

void mainForm::customEvent( QCustomEvent* p_event )
{
    // there's only one custom event that we generate.
    
    if( _live )
    {
        // this is a hack.  but it's better that allocating and destroying
        // an unsigned int; you have to use that much space for the
        // pointer anyway!

        unsigned int fader = (unsigned int)(p_event->data());

        if( fader == UINT_MAX )
            for( unsigned int i = 0; i < model->getNumFaders(); i++ )
                controller->modelLevelChanged( i, model->getFaderLevel( i ) );
        else
            controller->modelLevelChanged( fader, model->getFaderLevel(fader));
    }
}


void mainForm::cueContextMenu( QListViewItem * item, const QPoint & pos, int col )
{
    controller->cueContextMenu( item, pos );
}

void mainForm::editCue()
{
	extern Cue* g_context_cue;

	if( cueList->selectedItem() != NULL ) {
		QListViewItem* p_item =  cueList->selectedItem();
		Cue* cue = ((CueListItem*)p_item)->getCue();
		controller->editCue(cue);
	} else {
		if(g_context_cue) {
			//		printf("mainForm::editCue: found context cue\n");
			controller->editCue(g_context_cue);
		} else {
			printf("mainForm::editCue: no selected item\n");
		}
	}
}


void mainForm::deleteCue()
{
    MainController::deleteCue();
}

void mainForm::fadeToCue()
{
	extern Cue* g_context_cue;

	if( cueList->selectedItem() != NULL ) {
		QListViewItem* p_item =  cueList->selectedItem();
		Cue* cue = ((CueListItem*)p_item)->getCue();

//		printf("mainform::fadeToCue: found selected\n");
		controller->fadeToCue(cue);
	} else {
		if(g_context_cue) {
//			printf("mainForm::fadeToCue: found context cue %f\n", g_context_cue->number());
			controller->fadeToCue(g_context_cue);
		} else {
			printf("mainForm::editCue: no selected item\n");
		}
	}
}
