#ifndef _RTCFADER_H
#define _RTCFADER_H

#include <qvaluelist.h>

class DMXEngine;
class RTCFaderThread;

class RTCFader
{
    friend class RTCFaderThread;

    public:
      RTCFader( unsigned int faderNum, DMXEngine *engine );
      ~RTCFader();

      int getLevel();
      void setLevel( int p_level );
      void fadeTo( int p_level, float p_time );

      unsigned int number()
      {
          return m_faderNum;
      }

      void addChannel( unsigned int channel ) 
      {
          m_channels.append( channel );
      }

      void removeChannel( unsigned int channel )
      {
          m_channels.remove( channel );
      }

      void clearChannels()
      {
          m_channels.clear();
      }

      static void setEngine( DMXEngine *e )
      {
          m_engine = e;
      }

      bool isActive()
      {
          return m_active;
      }

    private:

      static int m_timerFrequency;   // timer freq in Hz (usu 64)

      static RTCFaderThread* m_thread;
      static DMXEngine* m_engine;

      void timer_tick();

      unsigned int m_faderNum;

      unsigned char m_level;
      unsigned char m_targetLevel;

      int m_d;
      int m_delta_d_above;
      int m_delta_d_below;
      int m_delta_y_above;
      int m_delta_y_below;

      bool m_active;

      QValueList<unsigned int> m_channels;

};


#endif // _RTCFADER_H
