
#ifndef _slmodel_h
#define _slmodel_h
#include <stdio.h>
#include <fstream>

using namespace std;

#include <qobject.h>
#include <qptrlist.h>
#include <qintdict.h>
#include <qtimer.h>

#define INITIAL_NUM_CHANNELS 40

//#include "cue.h"
//#include "dmxengine.h"
//#include "rtcfader.h"

class DMXEngine;
class RTCFader;
class Cue;



class SLModel : public QObject
{
    Q_OBJECT

    public:

    SLModel( DMXEngine* p_engine, QObject *parent = 0, const char* name = 0);
    ~SLModel();

    void checkpoint();
    void serialize( QDataStream& p_os );
    void toascii( FILE *fp);
    bool checkFile( QDataStream& p_is );
    void restore( QDataStream& p_is );
    void clear();

    unsigned int getNumCues();
    Cue* getCue( unsigned int p_index );

    void appendCue( Cue* p_cue );
    void insertCue( Cue* p_cue, unsigned int p_Index );
    void insertCue( Cue* p_cue, Cue* p_afterCue );
    void removeCue( Cue* p_cue );
    int findCue( Cue* p_cue );


    void goTo( Cue* p_cue );
    void fadeTo( Cue* p_cue );
    void fadeToCue( Cue* p_cue );

    bool isCurrent( Cue* p_cue );
    Cue* getCurrentCue();

    bool isRunning( Cue* p_cue );

    int getNumJumps( Cue* p_cue );
    
    int getFaderLevel( unsigned int p_fader );

    void mapChannelToFader( unsigned int p_channel, unsigned int p_fader );
    void removeChannelMap( unsigned int p_channel, unsigned int p_fader );

    // returns true if mapped, false if else.
    bool getFaderForChannel( unsigned int p_channel, unsigned int* p_fader );

    unsigned int getNumChannels();
    unsigned int getNumFaders();
    
    int getFaderForCue( Cue *p_cue, unsigned int p_fader );

    signals:

    void numFadersChanged( uint numFaders );
    void cueStatusChanged();
    
    public slots:

    void setNumChannels( uint p_channels );
    void setNumFaders( uint p_faders );
    void setFaderLevel( uint p_fader, int p_level );

    void fadeToNextCue();
    void reset();

    private slots:
    void dequeueRunningCue();

    private:

    QPtrList<Cue> m_cues;

    DMXEngine* m_engine;
    unsigned int m_numChannels;

    QPtrList<RTCFader> m_faders;
    QIntDict<RTCFader> m_faderMap;

    Cue* m_currCue;
    QPtrList<Cue> m_runningCues;

    QMap<Cue*, int> m_jumpCount;

    QTimer m_followTimer;
};

#endif
