#ifndef _CHANNEL_H
#define _CHANNEL_H

#include <qwidget.h> 
#include <qlayout.h>
#include <qevent.h>
#include <qdragobject.h>

enum ChannelState { CS_UP, CS_DOWN, CS_SAME, CS_CHANGED, CS_NONE };

class Channel : public QWidget
{
    Q_OBJECT

    public:

        Channel( QWidget *parent=0, const char *name=0 );

        void setChannelNum( uint num ) 
        { 
            _number = num; 
        }

        int getLevel()   
        { 
            return _level; 
        }

        int getLevelDisplay()
        {
            return _levelDisplay;
        }

        enum ChannelState getState()   
        { 
            return _state; 
        }

        bool isSelected()
        {
            return _selected;
        }

        QSizePolicy sizePolicy() const;
        QSize sizeHint() const;

        public slots:
        void setLevel( int level );
        void setLevelDisplay( int level );
        void revertLevelDisplay();
        void confirmLevelDisplay();

        void setState( enum ChannelState state );

        void setShowPercent( bool p )
        {
            _percent = p;
            revertLevelDisplay();
            update();
        }

        void setRecorded( bool r )
        {
            _recorded = r;
            update();
        }


        void select();
        void deselect();

    signals:
        void levelChanged( uint channel, int level );
        void stateChanged( uint channel, enum ChannelState );
        void selectStart( uint, bool retain = false );
        void selectEnter( uint );
        void selectEnd();
        void selected();
        void deselected();

    protected:
        void paintEvent( QPaintEvent * );
        void mousePressEvent ( QMouseEvent * e );
        void dragEnterEvent(QDragEnterEvent* event);
        void dropEvent(QDropEvent* event);


    private:

        uint _number;
        int _level;         //always DMX (eg, 0 to 255 )

        int _levelDisplay;  //always the integer to be displayed, either percent
        // or DMX

        enum ChannelState _state;
        bool _selected;
        bool _percent;
        bool _recorded;

};


#endif // _CHANNEL_H

