#include "channellayout.h"

class ChannelLayoutIterator : public QGLayoutIterator
{
public:
    ChannelLayoutIterator( QPtrList<QLayoutItem> *l )
        : idx( 0 ), list( l ) { }

    QLayoutItem *current()
    { return idx < int(list->count()) ? list->at(idx) : 0;  }

    QLayoutItem *next()
    { idx++; return current(); }

    QLayoutItem *takeCurrent()
    { return list->take( idx ); }

private:
    int idx;
    QPtrList<QLayoutItem> *list;
};


QLayoutIterator ChannelLayout::iterator()
{       
  return QLayoutIterator( new ChannelLayoutIterator(&list) );
}

void ChannelLayout::addItem( QLayoutItem *item )
{
    list.append( item );
}

ChannelLayout::~ChannelLayout()
{
    deleteAllItems();
}

QSize ChannelLayout::sizeHint() const
{
  return QSize( 50, 50 );
}

QSize ChannelLayout::minimumSize() const
{
  return QSize( 50, 50 );
}


void ChannelLayout::setGeometry( const QRect &rect )
{
  QLayout::setGeometry( rect );
  
  QPtrListIterator<QLayoutItem> it( list );
  if (it.count() == 0)
    return;
  
  QLayoutItem *o;

  QSize widget_size = o->sizeHint();
  int cols = rect.width() / widget_size.width();
  if( cols == 0 ) cols = 1;

  int curr_col = 0;
  int curr_row = 0;
  
  while ( (o = it.current()) != 0 ) 
    {
      ++it;

      if( curr_col == cols )
	{
	  curr_col = 0;
	  curr_row++;
	}
      
      int x = rect.x() + curr_col * (o->sizeHint().width() + spacing());
      int y = rect.y() + curr_row * (o->sizeHint().height() + spacing());
      
      o->setGeometry( QRect( QPoint( x, y ), o->sizeHint() ) );
      
      curr_col++;
      
    }
}









