#include <iostream>
using namespace std;

#include <string.h>

#include <qlayout.h>
#include <qscrollview.h>
#include <qpalette.h>
#include <qevent.h>
#include <qapplication.h>

#include "channel.h"
#include "channelpanel.h"

ChannelPanel::ChannelPanel( QWidget *parent, const char *name )
    : QScrollView( parent, name ), _flow( this ), _keySelect( false ),
    _keySelectRemove( false ), _keySelectNum( 0 ), _keySelectRange( false ),
_keySelectRangeStart(0)

{
    _numChannels = 0;
    _flow.setResizeMode( QLayout::FreeResize );
    viewport()->setPalette( QPalette( QColor( 0, 0, 0 ) ) );

    setSizePolicy( QSizePolicy( QSizePolicy::Expanding,
                QSizePolicy::Expanding ) ); 

    setHScrollBarMode( AlwaysOff );
    setDragAutoScroll( true );

    setShowPercent( true );

    setFocusPolicy(QWidget::ClickFocus);

    /*
       QSizePolicy size_pol( QSizePolicy::Expanding,
       QSizePolicy::Expanding, 1, 1 );
       _flow.mainWidget()->setSizePolicy( size_pol );
       addChild( _flow.mainWidget() );
       */

    //  setResizePolicy( Manual );
    //  enableClipper(true);

}

ChannelPanel::~ChannelPanel()
{
    for( unsigned int i = 0; i < _numChannels; i++ )
    {
        QWidgetItem* tmp = (QWidgetItem*)(_flow.removeItem());
        QWidget* tmpChan = tmp->widget();

        removeChild( tmpChan );
        delete tmpChan;
        delete tmp;
    }
}

void ChannelPanel::setNumChannels( uint num )
{
    if( _numChannels == num )
        return;

    hide();

    while( _numChannels != num )
        if( _numChannels < num )
        {
            char buff[30];
            sprintf( buff, "Channel %d", _numChannels );
            Channel* tmpChannel = new Channel( viewport(), buff );
            addChild( tmpChannel );

            connect( tmpChannel, SIGNAL(selectStart(uint, bool)),
                    this, SLOT(channelSelectStart(uint, bool)) );
            connect( tmpChannel, SIGNAL(selectEnter(uint)),
                    this, SLOT(channelSelectEnter(uint)));
            connect( tmpChannel, SIGNAL(selectEnd()),
                    this, SLOT(channelSelectEnd()));

            connect( tmpChannel, SIGNAL(levelChanged(uint, int)),
                    this, SIGNAL(levelChanged(uint, int)) );
            connect( tmpChannel, SIGNAL(stateChanged(uint, enum ChannelState)),
                    this, SIGNAL(stateChanged(uint, enum ChannelState)) );

            tmpChannel->setChannelNum( _numChannels );
            tmpChannel->setState( CS_NONE );
            tmpChannel->setShowPercent( _showPercent );

            _channelList.append( tmpChannel );
            _flow.addItem( new QWidgetItem( tmpChannel ) );
            _numChannels++;
        }
        else
        {	
            _channelList.removeLast();
            QWidgetItem* tmp = (QWidgetItem*)(_flow.removeItem());
            QWidget* tmpChan = tmp->widget();

            removeChild( tmpChan );
            delete tmpChan;
            delete tmp;

            _numChannels--;
        }

    setContentsPos(0,0);
    _flow.invalidate();
    _flow.activate();
    resizeContents( contentsWidth(), _flow.heightForWidth( contentsWidth() ) );
    updateGeometry();
    show();

    emit numChannelsChanged( _numChannels );

    return;
}

void ChannelPanel::setShowPercent( bool p )
{
    _showPercent = p;

    for( Channel* c = _channelList.first(); c; c = _channelList.next() )
        c->setShowPercent( p );
}

void ChannelPanel::select( unsigned int p_channel, bool p_selected )
{
    if( p_selected )
    {
        _channelList.at(p_channel)->select();
        if( _channelSelected.contains( p_channel ) == 0 )
            _channelSelected.push_back( p_channel );
    }
    else
    {
        _channelList.at(p_channel)->deselect();
        _channelSelected.remove( p_channel );
    }
}


void ChannelPanel::resizeEvent( QResizeEvent* e )
{
    QScrollView::resizeEvent(e);
    resizeContents( width(), _flow.heightForWidth( width() ) ); 
}


void ChannelPanel::channelSelectStart( uint channelNum, bool retain )
{
    _selectStart = channelNum;

    if( !retain )
        _channelSelected.clear();

    for( Channel* chan = _channelList.first(); 
            chan; chan = _channelList.next() )
        chan->revertLevelDisplay();

}

void ChannelPanel::channelSelectEnter( uint channelNum )
{
    unsigned int lo = QMIN( channelNum, _selectStart );
    unsigned int hi = QMAX( channelNum, _selectStart );

    unsigned int i = 0;
    for( Channel* chan = _channelList.first(); 
            chan; chan = _channelList.next() )
    {
        if( (i >= lo && i <= hi) || 
                _channelSelected.find(i) != _channelSelected.end() )
            chan->select();
        else 
            chan->deselect();

        i++;
    }
}

void ChannelPanel::channelSelectEnd()
{
    for( unsigned int i = 0; i < _channelList.count(); i++ )
        if( _channelList.at(i)->isSelected() )
            _channelSelected.push_back(i);
        else
            _channelSelected.remove(i);

    setFocus();

    _currLevel = 0;
}

void ChannelPanel::cancelSelection()
{
    for( QValueList<unsigned int>::Iterator i = _channelSelected.begin();
            i != _channelSelected.end(); i++ )
    {
        _channelList.at(*i)->revertLevelDisplay();
        _channelList.at(*i)->deselect();
    }

    _channelSelected.clear();
}

bool ChannelPanel::isAnySelected()
{
    for( unsigned int i = 0; i < _numChannels; i++ )
        if( _channelList.at(i)->isSelected() )
            return true;
    return false;
}

bool ChannelPanel::isSelected( unsigned int p_channel )
{
    return (_channelList.at(p_channel)->isSelected() );
}


void ChannelPanel::viewportResizeEvent( QResizeEvent* e )
{
    //verticalScrollBar()->setValue(0);
    setContentsPos(0,0);
    _flow.invalidate();
    _flow.activate();
    resizeContents( contentsWidth(), _flow.heightForWidth( contentsWidth() ) );
    updateGeometry();

    QScrollView::viewportResizeEvent( e );
}

void ChannelPanel::keyPressEvent( QKeyEvent* p_event )
{
    switch( p_event->key() )
    {
        case Key_PageUp:
            scrollBy( 0, -1 * visibleHeight() );
            break;
        case Key_PageDown:
            scrollBy( 0, visibleHeight() );
            break;
        case Key_Escape:
            if( isAnySelected() )
            {
                cancelSelection();
                return;
            }

            // it's important to ignore the event if we don't handle
            // it so the controller can handle it instead.

            p_event->ignore();
            break;

        case Key_Enter:  /*fallthrough intended */
        case Key_Return:
            for( QValueList<unsigned int>::Iterator i 
                    = _channelSelected.begin();
                    i != _channelSelected.end(); i++ )
            {
                _channelList.at(*i)->confirmLevelDisplay();
                _channelList.at(*i)->deselect();
                emit levelRecorded( *i, _channelList.at(*i)->getLevel() );
            }

            _channelSelected.clear();

            break;

        case Key_Plus:
            keySelectPressed( false );
            break;

        case Key_Minus:
            keySelectPressed( true );
            break;

        case Key_Slash:
            _keySelectRange = true;
            _keySelectRangeStart = _keySelectNum;
            _keySelectNum = 0;
            break;

        case Key_Asterisk :
            if( _keySelectNum > 0 )
            {
                if( _keySelectRange )
                    selectRange( _keySelectRangeStart - 1, _keySelectNum - 1, 
                            !_keySelectRemove );
                else
                    select( _keySelectNum - 1, !_keySelectRemove );

                _currLevel = 0;
            }
            _keySelect = false;
            _keySelectRemove = false;
            _keySelectNum = 0;
            _keySelectRange = false;
            _keySelectRangeStart = 0;
            QApplication::beep();
            break;

        case Key_0:
            numberPressed(0);
            break;

        case Key_1:
            numberPressed(1);
            break;

        case Key_2:
            numberPressed(2);
            break;

        case Key_3:
            numberPressed(3);
            break;

        case Key_4:
            numberPressed(4);
            break;

        case Key_5:
            numberPressed(5);
            break;

        case Key_6:
            numberPressed(6);
            break;

        case Key_7:
            numberPressed(7);
            break;

        case Key_8:
            numberPressed(8);
            break;

        case Key_9:
            numberPressed(9);
            break;

        case Key_Backspace:
            numberPressed(-1);
            break;

        default:
            p_event->ignore();
            return;
    }

    return;
}


void ChannelPanel::numberPressed( int p_num )
{
    // digits pressed, 0 thu 10.  -1 indicates that "backspace" was
    // pressed

    if( _keySelect )
    {
        // we're in key-select mode.  

        _keySelectNum = _keySelectNum * 10 + p_num;

        if( p_num == -1 || _keySelectNum > numChannels() )
        {
            // error case

            _keySelect = false;
            _keySelectRemove = false;
            _keySelectNum = 0;
            _keySelectRange = false;
            _keySelectRangeStart = 0;

            QApplication::beep();
        }
    }
    else
    {
        // we're setting levels

        if( p_num != -1 )
            if( levelVerify( _currLevel * 10 + p_num ) )
                _currLevel = _currLevel * 10 + p_num;
            else return;
        else
            _currLevel = _currLevel / 10;

        for( QValueList<unsigned int>::Iterator i = _channelSelected.begin();
                i != _channelSelected.end(); i++ )
            _channelList.at(*i)->setLevelDisplay( _currLevel );
    }


}

void ChannelPanel::keySelectPressed( bool p_isMinus )
{
    if( !_keySelect )
    {
        // enter key-select mode
        _keySelect = true;
        _keySelectRemove = p_isMinus;
        return;
    }
    else
    {
        if( _keySelectNum > 0 )
        {
            if( _keySelectRange )
                selectRange( _keySelectRangeStart - 1, _keySelectNum - 1, 
                        !_keySelectRemove );
            else
                select( _keySelectNum - 1, !_keySelectRemove );

            _currLevel = 0;
        }

        _keySelectRemove = p_isMinus;
        _keySelectNum = 0;
        _keySelectRange = false;
        _keySelectRangeStart = 0;
    }


}



void ChannelPanel::wheelEvent( QWheelEvent* p_event )
{
	if(!isAnySelected()) {
		p_event->ignore();
		return;
	}

	int levdelta = p_event->delta()/120 * 2;
//	printf("accept wheel %d: ", levdelta);
	
	Channel *ch;
	
	for( QValueList<unsigned int>::Iterator i = _channelSelected.begin();
	     i != _channelSelected.end(); i++ ) {

		ch = _channelList.at(*i);
		int oldlevel = ch->getLevelDisplay();

		int newlevel = oldlevel + levdelta;
		int maxlevel;
		if(newlevel < 0)
			newlevel = 0;
		else {
			if( _showPercent)
				maxlevel = 100;
			else
				maxlevel = 255;
			if(newlevel > maxlevel)
				newlevel = maxlevel;
		}

//		printf(" %03d,%03d", oldlevel, newlevel);
		ch->setLevelDisplay( newlevel );
		ch->confirmLevelDisplay();
                emit levelRecorded( *i, ch->getLevel() );
	}
//	printf("\n");
}
