
#ifndef _CHANNELPANEL_H
#define _CHANNELPANEL_H

#include <qscrollview.h>

#include <iostream>
using namespace std;

#include "channel.h"
#include "flow.h"

class ChannelPanel : public QScrollView
{
    Q_OBJECT
    Q_PROPERTY( uint numChannels READ numChannels WRITE setNumChannels )
    Q_PROPERTY( bool showPercent READ showPercent WRITE setShowPercent )

    friend class Channel;

    public:
    ChannelPanel( QWidget *parent=0, const char *name=0 );
    ~ChannelPanel();

    uint numChannels() const
    {
        return _numChannels;
    }

    int getChannelLevel( uint channel )
    {
        return _channelList.at(channel)->getLevel();
    }

    enum ChannelState getChannelState( uint channel )
    {
        return _channelList.at(channel)->getState();
    }

    bool showPercent() const
    {
        return _showPercent;
    }

    void select( unsigned int p_channel, bool p_selected );

    void selectRange( unsigned int p_start, 
            unsigned int p_end, bool p_selected )
    {
        for( unsigned int i = p_start; i <= p_end; i++ )
            select( i, p_selected );
    }

    bool isSelected( unsigned int p_channel );
    bool isAnySelected();

    void keyPress( QKeyEvent* e )
    {
        return keyPressEvent( e );
    }

    public slots:

    void setNumChannels( uint num );

    void setChannelLevel( uint channel, int level )
    {
        if( level != _channelList.at(channel)->getLevel() )
        {
            _channelList.at(channel)->setLevel(level);
            _channelList.at(channel)->update();
        }      
    }

    void setChannelState( uint channel, enum ChannelState state )
    {
        if( state != _channelList.at(channel)->getState() )
            _channelList.at(channel)->setState(state);
    }

    void setRecorded( uint channel, bool r )
    {
        _channelList.at(channel)->setRecorded(r);
    }

    void setShowPercent( bool p );

    void cancelSelection();


    signals:
    void numChannelsChanged( uint num );
    void levelChanged( uint channel, int level );
    void stateChanged( uint channel, enum ChannelState state );

    void levelRecorded( uint channel, int level );

    protected:
    void resizeEvent( QResizeEvent* e );

    protected slots:

    void keyPressEvent( QKeyEvent* e );
    void wheelEvent( QWheelEvent* e );
    void viewportResizeEvent( QResizeEvent* e );

    private slots:
    void channelSelectStart( uint channelNum, bool retain = false );
    void channelSelectEnter( uint channelNum );
    void channelSelectEnd();

    private:
    SimpleFlow _flow;
    unsigned int _numChannels;
    QPtrList<Channel> _channelList;
    QValueList<unsigned int> _channelSelected;
    bool _showPercent;
    unsigned int _selectStart;

    bool _keySelect;
    bool _keySelectRemove;
    unsigned int _keySelectNum;
    bool _keySelectRange;
    unsigned int _keySelectRangeStart;

    int _currLevel;

    bool levelVerify( int c )
    {
        if( _showPercent )
            return ( ( c >= 0 ) && ( c <= 100 ) );
        else
            return ( (c >= 0) && (c <= 255) );
    }

    void numberPressed( int p_num );
    void keySelectPressed( bool p_isMinus );
};


#endif //_CHANNELPANEL_H
