/****************************************************************************
** $Id: flow.h,v 1.1.1.1 2002/02/03 00:08:43 brian Exp $
**
** Definition of simple flow layout for custom layout example
**
** Created : 979899
**
** Copyright (C) 1997 by Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#ifndef FLOW_H
#define FLOW_H

#include <qlayout.h>
#include <qptrlist.h>

class SimpleFlow : public QLayout
{
public:
    SimpleFlow( QWidget *parent, int border=0, int space=-1,
                const char *name=0 )
        : QLayout( parent, border, space, name ),
        cached_width(0) {}
    SimpleFlow( QLayout* parent, int space=-1, const char *name=0 )
        : QLayout( parent, space, name ),
        cached_width(0) {}
    SimpleFlow( int space=-1, const char *name=0 )
        : QLayout( space, name ),
        cached_width(0) {}

    ~SimpleFlow();

    void addItem( QLayoutItem *item);
    QLayoutItem* removeItem();
    bool hasHeightForWidth() const;
    int heightForWidth( int ) const;
    QSize sizeHint() const;
    QSizePolicy::ExpandData expanding() const;
    QSize minimumSize() const;
    QLayoutIterator iterator();

protected:
    void setGeometry( const QRect& );

private:
    int doLayout( const QRect&, bool testonly = FALSE );
    QPtrList<QLayoutItem> list;
    int cached_width;
    int cached_hfw;

};


class SimpleFlowIterator :public QGLayoutIterator
{
public:
    SimpleFlowIterator( QPtrList<QLayoutItem> *l ) :idx(0), list(l)  {}
    uint count() const;
    QLayoutItem *current();
    QLayoutItem *next();
    QLayoutItem *takeCurrent();

private:
    int idx;
    QPtrList<QLayoutItem> *list;

};

#endif


